/*
  ControlLEDsBinary_Generic

  A simple example of communication from the Launchpad to the computer:
  the status of the 2 LEDs is sent out the serial port as binary values. We call this "serial"
  communication because the connection appears to both the Launchpad and the
  computer as a serial port, even though it may actually use
  a USB cable. Bytes are sent one after another (serially) from the Launchpad
  to the computer.

  You can use the Energia serial monitor to view the sent data, or it can
  be read by Processing or another program capable of reading
  data from a serial port.

  created 2006
  by David A. Mellis
  modified 9 Apr 2012
  by Tom Igoe and Scott Fitzgerald
  Modified 15 April 2013
  By Sean Alvarado
  Modified July 31 2017
  by Brian Cruickshank

  Hardware Required:
 * MSP-EXP430F5529 LaunchPad

  This example code is in the public domain.
 */

#define NUMBER 1 // take number / 2
#define MAX_JSON_STRLEN 40
int greenLedState = HIGH;
int redLedState = LOW;
uint32_t loopCtr = 0;
uint8_t i = 0;
char fromHost[MAX_JSON_STRLEN];
uint8_t numBytesRead = 0;
uint8_t j = 0;
uint8_t n = 0;
uint8_t buffer[6]; //loopCtr (4 bytes), LED status (1 byte), delimiter (1 byte)
boolean flag = false;
boolean running = true;
int debounceCounter = 0;

void setup() {
    pinMode(RED_LED, OUTPUT);
    pinMode(GREEN_LED, OUTPUT);
    Serial.setTimeout(6000); // 1s timeout when reading from serial port
    Serial.begin(9600);
    pinMode(PUSH2, INPUT_PULLUP);
    digitalWrite(RED_LED, redLedState);
    digitalWrite(GREEN_LED, greenLedState);


    Serial.flush();
}

void printDec(uint32_t ui){

    buffer[1] = (ui & 0x0F000) >> 12;
    buffer[2] = (ui & 0x00F00) >> 8;
    buffer[3] = (ui & 0x000F0) >> 4;
    buffer[4] = (ui & 0x0000F);
}

int getSerialString(){
    int i=0;
    char rxChar;
    boolean done = false;
    int waitCtr = 0;

    while (!done){
        if (Serial.available() > 0){
            rxChar = Serial.read();
            if (rxChar != -1){
                if (rxChar == (char)0x55){
                    done = true;

                } else {
                    fromHost[i++] = rxChar;
                }
            }
        } else {
            delayMicroseconds(100);     // pauses for 0.1 milliseconds
            waitCtr++;
            if (waitCtr > 2000) {
                done = true;
            }
        }
    }
    Serial.flush();
    return(i);

}
void loop() {
    if (running){

        numBytesRead = getSerialString();
        if (numBytesRead > 0){
            // Data from the host needs to be of the form 0x55 as delimiter followed by 0 to turn off
            // the red and green LEDs, 3 to turn on the red and green LEDs.
            // (Code is not generic at all, but it is small  )

            for (int j=0; j < numBytesRead; j++){
                switch(fromHost[j]){
                case 0: redLedState = LOW; greenLedState = LOW; break;
                case 1: redLedState = HIGH; greenLedState = LOW; break;
                case 2: redLedState = LOW; greenLedState = HIGH; break;
                case 3: redLedState = HIGH; greenLedState = HIGH; break;
                default: redLedState = LOW; greenLedState = LOW; break;
                }
            }
            digitalWrite(RED_LED, redLedState);
            digitalWrite(GREEN_LED, greenLedState);
            flag = true;
        }


        if (flag) {
            // send binary data to the host starting with a delimiter byte of 0x55, followed by
            // 4 bytes representing the current loopCtr value and 1 byte representing the 2 LEDs
            // (b0 = 1 if Red is on, b1 = 1 if Green is on)
            buffer[0] = 0x55;
            printDec(loopCtr++);
            buffer[5] = 0;
            if (redLedState &greenLedState) {
                buffer[5] = 3;
            } else {
                if (redLedState){
                    buffer[5] = 1;
                } else {
                    if (greenLedState){
                        buffer[5] = 2;
                    }
                }
            }
            Serial.write(buffer,6);


        }
        n++;
        if (n==NUMBER) {
            flag=true;
            n = 0;
        }
    }
    delay(40);

}




